/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail : 
 * LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

/*
 * This file was written by Gergely Nagy.
 * It contains the dialog box and the preferences handling routines.
 */
#include "common_headers.h"

static struct {
	GtkWidget *browser_entry;
	GtkWidget *tabpos_combo;
	GtkWidget *splash_checkbox;
	GtkWidget *terminal_entry;
	GtkWidget *lpr_entry;
	GtkWidget *show_tips;
	GtkWidget *max_recent_files;
} preferences_widgets;

static char *get_rc_filename (void);
static char *_get_value ( char *field, char *buffer );

/*
 * This is the callback function that retieves the
 * values set on the dialog box, and stores them
 * in the preferences struct.
 */
void callback_get_preferences ( GtkWidget *widget, gpointer data)
{
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [callback_get_preferences] start\n");
#endif

	/*
	 * Get the browser path ...
	 */
	free ( preferences.browser_command );
	preferences.browser_command = const2str (
		  gtk_entry_get_text ( GTK_ENTRY ( preferences_widgets.browser_entry ) ) );

	/*
	 * Get the terminal program...
	 */
	free ( preferences.terminal );
	preferences.terminal = const2str (
		  gtk_entry_get_text ( GTK_ENTRY ( preferences_widgets.terminal_entry ) ) );

	/*
	 * Get the printing command...
	 */
	free ( preferences.print_command );
	preferences.print_command = const2str (
		  gtk_entry_get_text ( GTK_ENTRY ( preferences_widgets.lpr_entry ) ) );

	/*
	 * Get the state of the splash window checkbox...
	 */
	if ( GTK_TOGGLE_BUTTON ( preferences_widgets.splash_checkbox ) -> active )
	{
		preferences.splash_screen = TRUE;
	} else {
		preferences.splash_screen = FALSE;
	}

	/*
	 * Get the state of the show tips at astartup checkbox...
	 */
	if ( GTK_TOGGLE_BUTTON ( preferences_widgets.show_tips ) -> active )
	{
		preferences.show_tips = TRUE;
	} else {
		preferences.show_tips = FALSE;
	}

	/*
	 * Get the value of the tabpos combobox...
	 */
	free ( preferences.tab_position );
	preferences.tab_position = const2str (
		  gtk_entry_get_text ( GTK_ENTRY ( GTK_COMBO ( preferences_widgets.tabpos_combo ) -> entry ) ) );

	/*
	 * Get the value of the max_recent_files spinbutton...
	 */
	preferences.max_recent_files = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON ( preferences_widgets.max_recent_files ) );

	/*
	 * Now do the destructive stuff...
	 */
	gtk_widget_destroy ( widget );

	apply_preferences ();

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [callback_get_preferences] end\n");
#endif
}

/*
 * Apply preferences ...
 */
void apply_preferences (void)
{
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [apply_preferences] start\n");
#endif
	if ( ! strcmp ( preferences.tab_position, _("Top") ) )
	{
		toggle_tab_pos ( NULL, 2 );
	}
	if ( ! strcmp ( preferences.tab_position, _("Bottom") ) )
	{
		toggle_tab_pos ( NULL, 3 );
	}
	if ( ! strcmp ( preferences.tab_position, _("Left") ) )
	{
		toggle_tab_pos ( NULL, 0 );
	}
	if ( ! strcmp ( preferences.tab_position, _("Right") ) )
	{
		toggle_tab_pos ( NULL, 1 );
	}

	recent_display ();

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [apply_preferences] end\n");
#endif
}

/*
 * This displays the preferences dialog.
 */
void display_preferences_dialog (void)
{

	GtkWidget *dialog_window;
	GtkWidget *main_vbox;

	GtkWidget *browser_frame;
	GtkWidget *browser_box;

	GtkWidget *button_box;
	GtkWidget *buttonOk;
	GtkWidget *buttonCancel;

	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *table;

	GList *tabpos_list=NULL;

	GtkWidget *misc_box;
	GtkWidget *misc_frame;

	GtkAdjustment *adj;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [display_preferences_dialog] start\n");
#endif

	/*
	 * Create the dialog window
	 */
	dialog_window = gtk_window_new ( GTK_WINDOW_DIALOG );

	gtk_window_set_title ( GTK_WINDOW ( dialog_window ), _("Preferences") );
	gtk_widget_set_usize ( dialog_window, 440, 360 );
	gtk_window_set_policy ( GTK_WINDOW ( dialog_window ), FALSE, TRUE, TRUE );
	gtk_window_set_modal ( GTK_WINDOW ( dialog_window ), TRUE );
	gtk_signal_connect ( GTK_OBJECT ( dialog_window ), "destroy", 
			     GTK_SIGNAL_FUNC ( gtk_widget_destroy ), dialog_window);

	gtk_container_set_border_width ( GTK_CONTAINER ( dialog_window ), 5 );

	main_vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( dialog_window ), main_vbox );

	/*
	 * Create the Browser Path frame
	 */
	browser_frame = gtk_frame_new ( _("Browser path") );
	gtk_box_pack_start ( GTK_BOX ( main_vbox ), browser_frame, FALSE, FALSE, 10 );

	browser_box = gtk_vbox_new ( FALSE, 1 );
	gtk_container_border_width ( GTK_CONTAINER ( browser_box ), 5 );
	gtk_container_add ( GTK_CONTAINER ( browser_frame ), browser_box );

	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( browser_box ), hbox );

	label = gtk_label_new ( _("Command to run external browser (%s will be replaced by the filename):") );
	gtk_label_set_justify ( GTK_LABEL ( label ), GTK_JUSTIFY_LEFT );
	gtk_box_pack_start ( GTK_BOX ( hbox ), label, FALSE, FALSE, 0 );
	
	preferences_widgets.browser_entry = gtk_entry_new ();
	gtk_entry_set_text ( GTK_ENTRY ( preferences_widgets.browser_entry ), 
			     preferences.browser_command );
	gtk_box_pack_start ( GTK_BOX ( browser_box ), preferences_widgets.browser_entry, FALSE, FALSE, 10 );

	/*
	 * Create the Other settings frame
	 */
	misc_frame = gtk_frame_new ( _("Other settings") );
	gtk_box_pack_start ( GTK_BOX ( main_vbox ), misc_frame, FALSE, FALSE, 10 );

	misc_box = gtk_vbox_new ( FALSE, 1 );
	gtk_container_border_width ( GTK_CONTAINER ( misc_box ), 5 );
	gtk_container_add ( GTK_CONTAINER ( misc_frame ), misc_box );

	/*
	 * Create the Tab position combobox (Other settings)
	 */
	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( misc_box ), hbox );

	label = gtk_label_new ( _("Tab position:") );
	gtk_label_set_justify ( GTK_LABEL ( label ), GTK_JUSTIFY_LEFT );
	gtk_box_pack_start ( GTK_BOX ( hbox ), label, FALSE, FALSE, 0 );

	preferences_widgets.tabpos_combo = gtk_combo_new ();

	tabpos_list = g_list_append ( tabpos_list, _("Top") );
	tabpos_list = g_list_append ( tabpos_list, _("Bottom") );
	tabpos_list = g_list_append ( tabpos_list, _("Left") );
	tabpos_list = g_list_append ( tabpos_list, _("Right") );
	gtk_combo_set_popdown_strings ( GTK_COMBO ( preferences_widgets.tabpos_combo ), tabpos_list );
	gtk_entry_set_editable ( GTK_ENTRY ( GTK_COMBO ( preferences_widgets.tabpos_combo ) -> entry ), FALSE );
	gtk_entry_set_text ( GTK_ENTRY ( GTK_COMBO ( preferences_widgets.tabpos_combo ) -> entry ),
			     preferences.tab_position );

	gtk_container_add ( GTK_CONTAINER ( misc_box ), preferences_widgets.tabpos_combo );

	/*
	 * Create the terminal inputbox (Other settings)
	 */
	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( misc_box ), hbox );

	label = gtk_label_new ( _("Terminal:") );
	gtk_label_set_justify ( GTK_LABEL ( label ), GTK_JUSTIFY_LEFT );
	gtk_box_pack_start ( GTK_BOX ( hbox ), label, FALSE, FALSE, 0 );

	preferences_widgets.terminal_entry = gtk_entry_new ();
	gtk_entry_set_text ( GTK_ENTRY ( preferences_widgets.terminal_entry ), 
			     preferences.terminal );
	gtk_box_pack_start ( GTK_BOX ( misc_box ), preferences_widgets.terminal_entry, FALSE, FALSE, 0 );

	/*
	 * Create the printing command inputbox (Other settings)
	 */
	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( misc_box ), hbox );

	label = gtk_label_new ( _("Command to print (%s will be replaced by the filename):") );
	gtk_label_set_justify ( GTK_LABEL ( label ), GTK_JUSTIFY_LEFT );
	gtk_box_pack_start ( GTK_BOX ( hbox ), label, FALSE, FALSE, 0 );

	preferences_widgets.lpr_entry = gtk_entry_new ();
	gtk_entry_set_text ( GTK_ENTRY ( preferences_widgets.lpr_entry ), 
			     preferences.print_command );
	gtk_box_pack_start ( GTK_BOX ( misc_box ), preferences_widgets.lpr_entry, FALSE, FALSE, 0 );

	/*
	 * Create a table at the bottom... (it looks nicer this way)
	 */
	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( misc_box ), hbox );

	table = gtk_table_new ( 2, 2, TRUE );
	gtk_container_add ( GTK_CONTAINER ( hbox ), table );

	/* 
	 * Create the show tips chechkbox (Other settings)
	 */
	preferences_widgets.show_tips = gtk_check_button_new_with_label ( _("Show tips at startup") );
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( preferences_widgets.show_tips ),
				       preferences.show_tips );
	gtk_table_attach ( GTK_TABLE ( table ), preferences_widgets.show_tips,
		0, 1, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 2 );

	/* 
	 * Create the Splash screen chechkbox (Other settings)
	 */
	preferences_widgets.splash_checkbox = gtk_check_button_new_with_label ( _("Splash screen") );
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( preferences_widgets.splash_checkbox ),
				       preferences.splash_screen );
	gtk_table_attach ( GTK_TABLE ( table ), preferences_widgets.splash_checkbox,
		0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 2 );

	/*
	 * Create the max number of recent files spinbutton...
	 */
	hbox = gtk_hbox_new ( FALSE, 0 );
	gtk_table_attach ( GTK_TABLE ( table ), hbox, 1, 2, 0, 1, 
			   GTK_FILL, GTK_FILL, 2, 2 );

	adj = (GtkAdjustment *) gtk_adjustment_new ( preferences.max_recent_files, 0.0, 12.0, 1.0, 5.0, 0.0 );
	preferences_widgets.max_recent_files = gtk_spin_button_new ( adj, 0, 0 );
	gtk_spin_button_set_wrap ( GTK_SPIN_BUTTON ( preferences_widgets.max_recent_files ), TRUE );
	gtk_box_pack_end ( GTK_BOX ( hbox ), preferences_widgets.max_recent_files, FALSE, TRUE, 5 );

	label = gtk_label_new ( _("Max. recent files:") );
	gtk_box_pack_end ( GTK_BOX ( hbox ), label, FALSE, TRUE, 5 );

	/*
	 * Create the button bar
	 */
	button_box = gtk_hbutton_box_new ();
	gtk_container_set_border_width ( GTK_CONTAINER ( button_box ), 10 );
	gtk_button_box_set_layout ( GTK_BUTTON_BOX ( button_box ), GTK_BUTTONBOX_END );
	gtk_button_box_set_spacing ( GTK_BUTTON_BOX ( button_box ), 10 );
	gtk_button_box_set_child_size ( GTK_BUTTON_BOX ( button_box ), 85, 20 );

	buttonOk = gtk_button_new_with_label ( _("Ok") );
	buttonCancel = gtk_button_new_with_label ( _("Cancel") );	

	gtk_container_add ( GTK_CONTAINER ( button_box ), buttonOk );
	gtk_container_add ( GTK_CONTAINER ( button_box ), buttonCancel );

	gtk_container_add ( GTK_CONTAINER ( main_vbox ), button_box );

	gtk_signal_connect_object ( GTK_OBJECT ( buttonOk ), "clicked", 
				    GTK_SIGNAL_FUNC ( callback_get_preferences ),
				    GTK_OBJECT ( dialog_window ) );

	gtk_signal_connect_object ( GTK_OBJECT ( buttonCancel ), "clicked",
				    GTK_SIGNAL_FUNC ( gtk_widget_destroy ),
				    GTK_OBJECT ( dialog_window ) );

	gtk_widget_show_all ( dialog_window );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [display_preferences_dialog] end\n");
#endif
}

/*
 * This should be called as soon as possible, for it sets
 * the default values.
 */
void init_preferences (void)
{
	int i;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [init_preferences] start\n");
#endif
	preferences.browser_command = const2str ( "/usr/bin/netscape %s" );
	preferences.terminal = const2str ( "xterm -T \"WebDesigner Shell\"" );
	preferences.tab_position = const2str ( _("Top") );
	preferences.print_command = const2str ( "lpr %s" );

	preferences.splash_screen = TRUE;
	preferences.show_tips = TRUE;

	preferences.max_recent_files = 4;

	for ( i = 0; i <= 10 ; i++ )
	{
		preferences.recent_files[i] = NULL;
	}

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [init_preferences] end\n");
#endif
}

/*
 * This frees up the memory allocated by init_preferences.
 * It is called just before exiting, and in some other situations...
 * (Eg. when resetting all the preferences)
 */
void done_preferences (void)
{
	int i;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] start\n");
#endif

	free ( preferences.browser_command );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] freed preferences.browser_command\n");
#endif
	free ( preferences.terminal );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] freed preferences.terminal\n");
#endif
	free ( preferences.print_command );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] freed preferences.print_command\n");
#endif
	free ( preferences.tab_position );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] freed preferences.tab_position\n");
#endif

	for ( i = 0; i <= 10; i++ )
	{
		if ( preferences.recent_files[i] != NULL )
#ifdef DEBUG
		{
#endif
			free ( preferences.recent_files[i] );
#ifdef DEBUG
			fprintf ( stderr, "(preferences.c) [done_preferences] freed preferences.recent_files[%d]\n", i );
		}
#endif
	}

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [done_preferences] end\n");
#endif
}

/*
 * This loads the configuration file.
 *
 * FIXME: loading of recent files is quite inefficient... takes
 * a lot of work and time.
 */
void load_preferences (void)
{
	FILE *preffile;
	char *filename;
	char *buffer=NULL;
	char *tmp=NULL;
	char *tmp2=NULL;
	int i;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [load_preferences] start\n");
#endif

	if ( ( filename = get_rc_filename () ) == NULL )
	{
#ifdef DEBUG
		fprintf (stderr, "(preferences.c) [load_preferences] end: load error\n");
#endif
		fprintf (stderr, ( _("Cannot load configuration file.") ) );
		return;
	}

	if ( ( preffile = fopen ( filename, "r" ) ) == NULL )
	{
		save_preferences ();
	} else {
		while ( !feof ( preffile ) )
		  {
		  	  buffer = get_full_line ( preffile );
			  if ( strstr ( buffer, "browser_path=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "browser_path", buffer ) ) != NULL )
				  {
				  	free ( preferences.browser_command );
					preferences.browser_command = const2str ( tmp );
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "tab_position=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "tab_position", buffer ) ) != NULL )
				  {
				  	free ( preferences.tab_position );
					preferences.tab_position = const2str ( _(tmp) );
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "splash_screen=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "splash_screen", buffer ) ) != NULL )
				  {
				  	preferences.splash_screen = atoi (tmp);
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "show_tips=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "show_tips", buffer ) ) != NULL )
				  {
				  	preferences.show_tips = atoi (tmp);
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "terminal=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "terminal", buffer ) ) != NULL )
				  {
				  	free ( preferences.terminal );
					preferences.terminal = const2str ( tmp );
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "print_command=") != NULL )
			    {
			    	if ( ( tmp = _get_value ( "print_command", buffer ) ) != NULL )
				  {
				  	free ( preferences.print_command );
					preferences.print_command = const2str ( tmp );
					free ( tmp );
				  }
			    }
			  if ( strstr ( buffer, "max_recent_files=" ) != NULL )
			    {
			    	if ( ( tmp = _get_value ( "max_recent_files", buffer ) ) != NULL )
				  {
				  	preferences.max_recent_files = atoi (tmp);
					free ( tmp );
				  }
			    }
			  for ( i = 0; i < 11 ; i++ )
			  {
			  	if ( ( tmp2 = (char *)malloc ( strlen ("recent_file_") + 16 ) ) == NULL )
				{
#ifdef DEBUG
					fprintf (stderr, "(preferences.c) [load_preferences] failed to allocate %i bytes of memory for tmp2\n", strlen("recent_file_") + 16 );
#endif
					break;
				}
#ifdef DEBUG
				fprintf (stderr, "(preferences.c) [load_preferences] allocated %i bytes of memory for tmp2\n", strlen("recent_file_") + 16 );
#endif
				sprintf(tmp2,"recent_file_%d=", i);
			  	if ( strstr ( buffer, tmp2 ) != NULL )
				{
					sprintf(tmp2,"recent_file_%d", i );
					if ( ( tmp = _get_value ( tmp2, buffer ) ) != NULL )
					{
						if ( preferences.recent_files[i] != NULL )
							free ( preferences.recent_files[i] );
						preferences.recent_files[i] = const2str ( tmp );
						free ( tmp );
					}
				}
				free ( tmp2 );
#ifdef DEBUG
				fprintf (stderr, "(preferences.c) [load_preferences] freed tmp2\n" );
#endif
			  }
			  free ( buffer );
		  }
		fclose ( preffile );
	}

	free ( filename );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [load_preferences] freed filename\n");
	fprintf (stderr, "(preferences.c) [load_preferences] end\n");
#endif
}

/*
 * This saves the preferences.
 */
void save_preferences (void)
{
	FILE *preffile;
	char *filename;
	char *my_tab_position="Top";
	int i;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [save_preferences] start\n");
#endif

	if ( ! strcmp ( preferences.tab_position, _("Top") ) )
	{
		my_tab_position="Top";
	}
	if ( ! strcmp ( preferences.tab_position, _("Bottom") ) )
	{
		my_tab_position="Bottom";
	}
	if ( ! strcmp ( preferences.tab_position, _("Left") ) )
	{
		my_tab_position="Left";
	}
	if ( ! strcmp ( preferences.tab_position, _("Right") ) )
	{
		my_tab_position="Right";
	}

	if ( ( filename = get_rc_filename () ) == NULL )
	{
		merror ( _("Cannot save configuration file.") );
		return;
	}

	if ( ( preffile = fopen ( filename, "w" ) ) == NULL )
	{
		merror ( _("Cannot save configuration file.") );
		return;
	}
	fprintf ( preffile, _("# WebDesigner configuration file.\n\n" ) );
	fprintf ( preffile, "browser_path=%s\ntab_position=%s\nsplash_screen=%d\nterminal=%s\n",
		  preferences.browser_command, my_tab_position, preferences.splash_screen,
		  preferences.terminal );
	fprintf ( preffile, "print_command=%s\nshow_tips=%d\nmax_recent_files=%d\n",
		  preferences.print_command, preferences.show_tips, preferences.max_recent_files );

	for ( i = 0; i < 11 ; i++ )
	{
		if ( preferences.recent_files[i] != NULL )
		{
			fprintf( preffile, "recent_file_%d=%s\n", i, preferences.recent_files[i] );
		}
	}

	fclose ( preffile );
	free ( filename );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [save_preferences] freed filename\n");
	fprintf (stderr, "(preferences.c) [save_preferences] end\n");
#endif
}

/*
 * This returns the rc filename...
 */
char *get_rc_filename (void)
{
	char *filename;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [get_rc_filename] start\n");
#endif

	if ( ! getenv ("HOME") )
	{
#ifdef DEBUG
		fprintf (stderr, "(preferences.c) [get_rc_filename] end: HOME is not set\n");
#endif
		return NULL;
	}

	filename = (char *)malloc (strlen(getenv("HOME"))+strlen("/.wdesignrc")+16);
#ifdef DEBUG
	fprintf ( stderr, "(preferences.c) [get_rc_filename] allocated %d bytes memory for filename\n", strlen(getenv("HOME"))+strlen("/.wdesignrc")+16);
#endif
	if ( getenv ("HOME") )
	{
		sprintf ( filename, "%s%s", getenv("HOME"), "/.wdesignrc" );
	}
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [get_rc_filename] end\n");
#endif
	return filename;
}

/*
 * This one expects a field=value in the buffer...
 * FIXME: Cannot handle extra whitespaces yet. No comment handling, etc.
 */
char *_get_value (char *field, char *buffer)
{
	char *tmp;
	char *tmp2;

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] start\n");
#endif

	if ( !buffer || !field )
#ifdef DEBUG
	{
		fprintf (stderr,"(preferences.c) [_get_value] end: invalid arguments\n");
#endif
		return NULL;
#ifdef DEBUG
	}
#endif

	/* 
	 * Allocate memory 
	 */
  	if ( ( tmp = (char *)malloc ( strlen ( buffer ) ) ) == NULL )
	{
		merror (_("Not enough memory!"));
#ifdef DEBUG
		fprintf (stderr, "(preferences.c) [_get_value] failed to allocate %d bytes memory for tmp\n", strlen(buffer) );
		fprintf (stderr,"(preferences.c) [_get_value] end: not enough memory\n");
#endif
		return NULL;
	}
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] allocated %d bytes memory for tmp\n", strlen(buffer));
#endif

	if ( ( tmp2 = (char *)malloc (strlen(field)+16) ) == NULL )
	{
		merror (_("Not enough memory!"));
#ifdef DEBUG
		fprintf (stderr, "(preferences.c) [_get_value] failed to allocate %d bytes memory for tmp2\n", strlen(field)+16 );
		fprintf (stderr,"(preferences.c) [_get_value] end: not enough memory\n");
#endif
		g_free ( tmp );
		return NULL;
	}
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] allocated %d bytes memory for tmp2\n", strlen(field)+16);
#endif
	/*
	 * tmp2 should be field=%s, that is what we're
	 * searching for in buffer...
	 */
	sprintf ( tmp2, "%s=%%s", field );
	if ( ( sscanf ( buffer, tmp2, tmp ) ) != 1 )
	  {
	  	/*
		 * Well... the contents of the buffer wasn't in the expected
		 * format, so we return NULL. load_preferences notices it, and
		 * leaves the current field untouched.
		 */
		g_free ( tmp );
		g_free ( tmp2 );
	  	return NULL;
	  }

	/*
	 * tmp2 can now be freed...
	 */
	free ( tmp2 );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] freed tmp2\n");
#endif

	/*
	 * ... but only for a little while, we need it again ...
	 */
	if ( ( tmp2 = (char *)malloc (strlen(buffer)-strlen(field)) ) == NULL )
	{
		merror (_("Not enough memory!"));
#ifdef DEBUG
		fprintf (stderr, "(preferences.c) [_get_value] failed to allocate %d bytes memory for tmp2\n", strlen(buffer)-strlen(field) );
		fprintf (stderr,"(preferences.c) [_get_value] end: not enough memory\n");
#endif
		g_free ( tmp );
		return NULL;
	}

#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] allocated %d bytes memory for tmp2\n", strlen(buffer)-strlen(field));
#endif
	
	/*
	 * Now copy the string after field= to tmp2 ...
	 */
  	strncpy ( tmp2, strstr(buffer, tmp), strlen(buffer)-strlen(field) );

	/*
	 * And now free the memory...
	 */
	free ( tmp );
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] freed tmp\n");
#endif
#ifdef DEBUG
	fprintf (stderr, "(preferences.c) [_get_value] end\n");
#endif
	return tmp2;
}
